<?php
/**
 * @file
 * Contains LakeDrops\Drupal8Theme\SASS\Plugin.
 */

namespace LakeDrops\Drupal8Theme\SASS;

use Composer\Composer;
use Composer\EventDispatcher\EventSubscriberInterface;
use Composer\IO\IOInterface;
use Composer\Plugin\PluginInterface;
use Composer\Script\ScriptEvents;

/**
 * Composer plugin for handling drupal sass theme template.
 */
class Plugin implements PluginInterface, EventSubscriberInterface
{

  /**
   * @var \LakeDrops\Drupal8Theme\SASS\Handler
   */
  protected $handler;

  /**
   * {@inheritdoc}
   */
  public function activate(Composer $composer, IOInterface $io)
  {
    $this->handler = new Handler($composer, $io);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents()
  {
    return array(
      ScriptEvents::POST_INSTALL_CMD => 'initThemeEvent',
      ScriptEvents::POST_UPDATE_CMD => 'updateThemeEvent',
    );
  }

  /**
   * Post package install event callback.
   *
   * @param \Composer\Script\Event $event
   */
  public function initThemeEvent($event)
  {
    $this->handler->updateTheme($event);
  }

  /**
   * Post package update event callback.
   *
   * @param \Composer\Script\Event $event
   */
  public function updateThemeEvent($event)
  {
    $this->handler->setUpdate(TRUE);
    $this->handler->updateTheme($event);
  }

  /**
   * Script callback for initializing the theme.
   *
   * @param \Composer\Script\Event $event
   * @param bool $update
   * @param bool $reset
   * @param bool $overwrite
   */
  public static function init($event, $update = FALSE, $reset = FALSE, $overwrite = FALSE)
  {
    $handler = new Handler($event->getComposer(), $event->getIO(), $update, $reset, $overwrite);
    $handler->updateTheme($event);
  }

  /**
   * Script callback for updating the theme.
   *
   * @param \Composer\Script\Event $event
   */
  public static function update($event)
  {
    self::init($event, TRUE);
  }

  /**
   * Script callback for resetting the theme.
   *
   * @param \Composer\Script\Event $event
   */
  public static function reset($event)
  {
    self::init($event, FALSE, TRUE);
  }

  /**
   * Script callback for updating the theme and overwriting existing custom files
   *
   * @param \Composer\Script\Event $event
   */
  public static function overwrite($event)
  {
    self::init($event, FALSE, FALSE, TRUE);
  }

}
